class MERTimeAllocation(PWReportPage):
	def init(self):
		super(MERTimeAllocation, self).init()
		self.project				= None
		self.projectRateFormatter   = None
		self.plannedWorkFormatter   = None
		self.actualWorkFormatter    = None		
		self.withObjective			= True
		self.contentMode			= 0
		self.withTotals			    = 1
		self.iterIndex				= 0
		self.deeperIterIndex        = 0
		self.levelDepth             = 10
		self.withAssignments        = False
		self.iterPhase				= None
		self.iterRes				= None
		self.iterItem				= None
		self.iterElement 			= None
		self.iterPhaseCostIndex	    = 0
		self.iterPhaseCost			= None
		self.iterPhaseResIndex		= 0
		self.infoElementsSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("informationDate", True)]
		self.eventsElementsSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("startDate", True)]
		self.masterResourcesSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		self.iterPhaseRes			= None
		return self
		
	def rateFormatter(self):
		if self.projectRateFormatter == None:
			self.projectRateFormatter = MEResource.formatterForPropertyWithKey_project_("standardRate",self.project)
		return self.projectRateFormatter

	def plannedWorkFormatter(self):
		if self.plannedWorkFormatter == None:
			self.plannedWorkFormatter = MEAssignment.formatterForPropertyWithKey_project_("plannedWork",self.project)
		return self.plannedWorkFormatter


	def showPlannedValues(self):
		return self.contentMode == 0 
		
	def showActualValues(self):
		return self.contentMode == 1 		

	def showObjective(self):
		return self.project.itemDescription()

	def topPhases(self):
		activities = self.project.visibleSubActivitiesArray()
		startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("plannedStartDate", True)
		if activities:
			return activities.sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()
	
	def addActivitiesAndChildActivitiesDeep(self, activities, result):
		if activities:
			for aAct in activities:
				if ((self.levelDepth == 0) or (aAct.level() <= self.levelDepth)) :
					if self.withAssignments:
						result.append(aAct)
					else:
						if aAct.isAssignment() == 0:
							result.append(aAct)
					self.addActivitiesAndChildActivitiesDeep(aAct.visibleSubActivitiesArray(), result)
		return	

	def getActsWithGroups(self):
		result = []
		activities = self.project.visibleSubActivitiesArray()
		if activities:
			startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
			self.addActivitiesAndChildActivitiesDeep(activities, result)	
			return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()

	def remainingWorkperItem(self):
		remaining = 0
		actual = self.iterItem.actualWork()
		expected = self.iterItem.expectedWork()
		if expected != None:
			if actual != None:
				actual = actual.negatedDuration()	
				remaining = expected.durationByAddingDuration_defaults_roundParts_deriveFromDuration_(actual, self.project.durationDefaults(), -1, None)
			else: 
				remaining = expected
			return remaining
	
	def theUnit(self):
		return self.project.durationDefaults()
	
	def resourceStandardRate(self):
		result = 0
		res = self.iterRes
		if res != None:
			rate = res.valueForKey_("standardRate")
			if rate != None:
				aamount = rate.valueForKey_("amount")
				if aamount != None:
					result = aamount
		return result


	def workPerPhase(self, property):
		amountPerResource = {}
		result = NSMutableArray.array()
		assignments = self.iterPhase.valueForKey_("assignmentsDeep")
		for anassignment in assignments:
			atitle = anassignment.valueForKey_("title")
			aamount = anassignment.valueForKey_(property)
			if aamount:
				oldAmount = amountPerResource.get(atitle)
				if oldAmount == None:
					amountPerResource[atitle] = aamount
				else:
					amountPerResource[atitle] =  aamount.durationByAddingDuration_defaults_roundParts_deriveFromDuration_(oldAmount, self.project.durationDefaults(), -1, None)		
		for key in amountPerResource.keys():
			dict = {}
			dict["name"] = key
			dict["value"] = amountPerResource[key]
			result.append(dict)
		byName = NSSortDescriptor.alloc().initWithKey_ascending_("name", True)
		return result.sortedArrayUsingDescriptors_([byName])

	def workPerItem(self, property):
		amountPerResource = {}
		result = NSMutableArray.array()
		assignments = self.iterItem.valueForKey_("assignmentsDeep")
		for anassignment in assignments:
			atitle = anassignment.valueForKey_("title")
			aamount = anassignment.valueForKey_(property)
			if aamount:
				oldAmount = amountPerResource.get(atitle)
				if oldAmount == None:
					amountPerResource[atitle] = aamount
				else:
					amountPerResource[atitle] =  aamount.durationByAddingDuration_defaults_roundParts_deriveFromDuration_(oldAmount, self.project.durationDefaults(), -1, None)		
		for key in amountPerResource.keys():
			dict = {}
			dict["name"] = key
			dict["value"] = amountPerResource[key]
			result.append(dict)
		byName = NSSortDescriptor.alloc().initWithKey_ascending_("name", True)
		return result.sortedArrayUsingDescriptors_([byName])


	def masterResources(self):
		sortedMasterResources = self.project.resources()
		sortedMasterResources = sortedMasterResources.sortedArrayUsingDescriptors_(self.masterResourcesSortDescriptors)
		return sortedMasterResources


	def getWorkOfMasterResource(self, property):
		aWork = None
		work = None
		resource = self.iterRes
		if resource != None:
			resName = resource.title()
			assignments = self.iterItem.valueForKey_("assignmentsDeep")
			if assignments != None:
				for anassignment in assignments:
					atitle = anassignment.valueForKey_("title")
					if atitle == resName:
						aWork = anassignment.valueForKey_(property)
						if work == None:
							work = aWork
						else:
							work =  work.durationByAddingDuration_defaults_roundParts_deriveFromDuration_(aWork, self.project.durationDefaults(), -1, None)							
				if work != None:
					return "%1.2f" % (work.manDaysWithDefaults_(self.project.durationDefaults()))
				else:
					return work
					
	def plannedWorkInManDays(self):
		if self.iterItem.plannedWork() != None:
			return "%1.2f" % (self.iterItem.plannedWork().manDaysWithDefaults_(self.project.durationDefaults()))

	def actualWorkInManDays(self):
		if self.iterItem.actualWork() != None:
			return "%1.2f" % (self.iterItem.actualWork().manDaysWithDefaults_(self.project.durationDefaults()))

	def iterResPlannedWork(self):
		if self.iterRes.plannedWork() != None:
			return "%1.2f" % (self.iterRes.plannedWork().manDaysWithDefaults_(self.project.durationDefaults()))
				
	def spaceByLevel(self):
		many = self.iterItem.level()		
		return many*15
	
	def showResourceWorkInThisRow(self):
		if self.iterItem.level() > 1:	
			return True
		else:
			return False
			
		
		
	
	def ResoureWorkPerPhase(self):
		return self.workPerPhase("plannedWork")

	def ResoureActualWorkPerPhase(self):
		return self.workPerPhase("actualWork")


	def ResoureWorkPerItem(self):
		return self.workPerItem("plannedWork")

	def ResoureActualWorkPerItem(self):
		return self.workPerItem("actualWork")


	def masterResoureWorkPerItem(self):
		return self.getWorkOfMasterResource("plannedWork")

	def masterResoureActualWorkPerItem(self):
		return self.getWorkOfMasterResource("actualWork")


	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def totalRowClass(self):
		if len(self.topPhases())%2:
			return "even"

	def dateFormat(self):
		return "medium/short"


	def cellClassForAmount(self, amount):
		if amount < 0:
			return "negativeBalance"
		else:
			return "positiveBalance"


